Class: NewRelic::Agent::OpenTelemetry::GenericTranslator
- Inherits:
-
BaseTranslator
- Object
- BaseTranslator
- NewRelic::Agent::OpenTelemetry::GenericTranslator
- Defined in:
- lib/new_relic/agent/opentelemetry/translators/generic_translator.rb
Overview
The Generic translator will sends all attributes to custom attributes because there are no known New Relic attributes to translate
Constant Summary
Constants included from AttributeMappings
AttributeMappings::DATASTORE_MAPPINGS, AttributeMappings::DEFAULT_DESTINATIONS, AttributeMappings::HTTP_CLIENT_MAPPINGS, AttributeMappings::HTTP_SERVER_MAPPINGS, AttributeMappings::MESSAGING_CONSUMER_MAPPINGS, AttributeMappings::MESSAGING_PRODUCER_MAPPINGS, AttributeMappings::REDIS_MAPPINGS, AttributeMappings::RPC_CLIENT_MAPPINGS, AttributeMappings::RPC_SERVER_MAPPINGS
Class Method Summary collapse
Methods inherited from BaseTranslator
add_specialized_attributes, translate
Class Method Details
.mappings_hash(_kind) ⇒ Object
14 15 16 |
# File 'lib/new_relic/agent/opentelemetry/translators/generic_translator.rb', line 14 def mappings_hash(_kind) NewRelic::EMPTY_HASH end |