Module: NewRelic::Agent
- Extended by:
- Forwardable, Agent, SupportabilityHelper
- Included in:
- Agent
- Defined in:
- lib/new_relic/agent.rb,
lib/new_relic/agent/method_tracer.rb,
lib/new_relic/agent/distributed_tracing.rb,
lib/new_relic/agent/external.rb,
lib/new_relic/agent/instrumentation/controller_instrumentation.rb,
lib/new_relic/agent/datastores.rb,
lib/new_relic/agent/messaging.rb,
lib/new_relic/agent/sql_sampler.rb,
lib/new_relic/agent/tracer.rb,
lib/new_relic/agent/transaction.rb,
lib/new_relic/agent/transaction_sampler.rb,
lib/new_relic/agent/transaction/external_request_segment.rb
Overview
This module contains most of the public API methods for the Ruby Agent.
For adding custom instrumentation to method invocations, see the docs for MethodTracer and MethodTracer::ClassMethods.
For information on how to trace transactions in non-Rack contexts, see Instrumentation::ControllerInstrumentation.
For general documentation about the Ruby agent, see: docs.newrelic.com/docs/agents/ruby-agent
Defined Under Namespace
Modules: Datastores, DistributedTracing, External, Instrumentation, Messaging, MethodTracer Classes: SqlSampler, Tracer, Transaction, TransactionSampler
Recording custom metrics collapse
-
#increment_metric(metric_name, amount = 1) ⇒ Object
Increment a simple counter metric.
-
#record_metric(metric_name, value) ⇒ Object
Record a value for the given metric name.
Recording custom errors collapse
-
#ignore_error_filter(&block) ⇒ Object
Set a filter to be applied to errors that the Ruby Agent will track.
-
#notice_error(exception, options = {}) ⇒ Object
Send an error to New Relic.
-
#set_error_group_callback(callback_proc) ⇒ Object
Set a callback proc for determining an error’s error group name.
Recording custom Insights events collapse
-
#record_custom_event(event_type, event_attrs) ⇒ Object
Record a custom event to be sent to New Relic Insights.
Manual agent configuration and startup/shutdown collapse
-
#add_instrumentation(file_pattern) ⇒ Object
Add instrumentation files to the agent.
-
#after_fork(options = {}) ⇒ Object
Register this method as a callback for processes that fork jobs.
-
#drop_buffered_data ⇒ Object
Clear out any data the agent has buffered but has not yet transmitted to the collector.
-
#manual_start(options = {}) ⇒ Object
Call this to manually start the Agent in situations where the Agent does not auto-start.
-
#require_test_helper ⇒ Object
Require agent testing helper methods.
-
#set_sql_obfuscator(type = :replace, &block) ⇒ Object
This method sets the block sent to this method as a sql obfuscator.
-
#shutdown(options = {}) ⇒ Object
Shutdown the agent.
Ignoring or excluding data collapse
-
#disable_all_tracing ⇒ Object
Yield to the block without collecting any metrics or traces in any of the subsequent calls.
-
#disable_sql_recording ⇒ Object
This method sets the state of sql recording in the transaction sampler feature.
-
#ignore_apdex ⇒ Object
This method disables the recording of Apdex metrics in the current transaction.
-
#ignore_enduser ⇒ Object
This method disables browser monitoring javascript injection in the current transaction.
-
#ignore_transaction ⇒ Object
This method disables the recording of the current transaction.
Adding custom attributes to traces collapse
-
#add_custom_attributes(params) ⇒ Object
Add attributes to the transaction trace, Insights Transaction event, and any traced errors recorded for the current transaction.
-
#add_custom_span_attributes(params) ⇒ Object
Add custom attributes to the span event for the current span.
-
#set_user_id(user_id) ⇒ Object
Set the user id for the current transaction.
Transaction naming collapse
-
#get_transaction_name ⇒ Object
Get the name of the current running transaction.
-
#set_transaction_name(name, options = {}) ⇒ Object
Set the name of the current running transaction.
Trace and Entity metadata collapse
-
#linking_metadata ⇒ Object
Returns a new hash containing trace and entity metadata that can be used to relate data to a trace or to an entity in APM.
Manual browser monitoring configuration collapse
-
#browser_timing_header(nonce = nil) ⇒ Object
This method returns a string suitable for inclusion in a page - known as ‘manual instrumentation’ for Real User Monitoring.
Instance Method Details
#add_custom_attributes(params) ⇒ Object
Add attributes to the transaction trace, Insights Transaction event, and any traced errors recorded for the current transaction.
If Browser Monitoring is enabled, and the browser_monitoring.attributes.enabled configuration setting is true, these custom attributes will also be present in the script injected into the response body, making them available on Insights PageView events.
606 607 608 609 610 611 612 613 614 615 616 617 618 619 |
# File 'lib/new_relic/agent.rb', line 606 def add_custom_attributes(params) # THREAD_LOCAL_ACCESS record_api_supportability_metric(:add_custom_attributes) if params.is_a?(Hash) Transaction.tl_current&.add_custom_attributes(params) segment = ::NewRelic::Agent::Tracer.current_segment if segment add_new_segment_attributes(params, segment) end else ::NewRelic::Agent.logger.warn("Bad argument passed to #add_custom_attributes. Expected Hash but got #{params.class}") end end |
#add_custom_span_attributes(params) ⇒ Object
Add custom attributes to the span event for the current span. Attributes will be visible on spans in the New Relic Distributed Tracing UI and on span events in New Relic Insights.
Custom attributes will not be transmitted when high_security setting is enabled or custom_attributes setting is disabled.
640 641 642 643 644 645 646 647 648 649 650 |
# File 'lib/new_relic/agent.rb', line 640 def add_custom_span_attributes(params) record_api_supportability_metric(:add_custom_span_attributes) if params.is_a?(Hash) if segment = NewRelic::Agent::Tracer.current_segment segment.add_custom_attributes(params) end else ::NewRelic::Agent.logger.warn("Bad argument passed to #add_custom_span_attributes. Expected Hash but got #{params.class}") end end |
#add_instrumentation(file_pattern) ⇒ Object
Add instrumentation files to the agent. The argument should be a glob matching ruby scripts which will be executed at the time instrumentation is loaded. Since instrumentation is not loaded when the agent is not running it’s better to use this method to register instrumentation than just loading the files directly, although that probably also works.
468 469 470 471 |
# File 'lib/new_relic/agent.rb', line 468 def add_instrumentation(file_pattern) record_api_supportability_metric(:add_instrumentation) NewRelic::Control.instance.add_instrumentation(file_pattern) end |
#after_fork(options = {}) ⇒ Object
Register this method as a callback for processes that fork jobs.
If the master/parent connects to the agent prior to forking the agent in the forked process will use that agent_run. Otherwise the forked process will establish a new connection with the server.
Use this especially when you fork the process to run background jobs or other work. If you are doing this with a web dispatcher that forks worker processes then you will need to force the agent to reconnect, which it won’t do by default. Passenger and Unicorn are already handled, nothing special needed for them.
Options:
-
:force_reconnect => trueto force the spawned process to establish a new connection, such as when forking a long running process. The default is false–it will only connect to the server if the parent had not connected. -
:keep_retrying => falseif we try to initiate a new connection, this tells me to only try it once so this method returns quickly if there is some kind of latency with the server.
431 432 433 434 435 |
# File 'lib/new_relic/agent.rb', line 431 def after_fork( = {}) record_api_supportability_metric(:after_fork) # the following line needs else branch coverage agent.after_fork() if agent # rubocop:disable Style/SafeNavigation end |
#browser_timing_header(nonce = nil) ⇒ Object
This method returns a string suitable for inclusion in a page - known as ‘manual instrumentation’ for Real User Monitoring. Can return either a script tag with associated javascript, or in the case of disabled Real User Monitoring, an empty string
This is the header string - it should be placed as high in the page as is reasonably possible - that is, before any style or javascript inclusions, but after any header-related meta tags
In previous agents there was a corresponding footer required, but all the work is now done by this single method.
808 809 810 811 812 813 814 |
# File 'lib/new_relic/agent.rb', line 808 def browser_timing_header(nonce = nil) record_api_supportability_metric(:browser_timing_header) return EMPTY_STR unless agent agent.javascript_instrumentor.browser_timing_header(nonce) end |
#disable_all_tracing ⇒ Object
Yield to the block without collecting any metrics or traces in any of the subsequent calls. If executed recursively, will keep track of the first entry point and turn on tracing again after leaving that block. This uses the thread local Tracer::State.
544 545 546 547 548 549 550 551 552 553 554 555 |
# File 'lib/new_relic/agent.rb', line 544 def disable_all_tracing record_api_supportability_metric(:disable_all_tracing) return yield unless agent begin agent.push_trace_execution_flag(false) yield ensure agent.pop_trace_execution_flag end end |
#disable_sql_recording ⇒ Object
This method sets the state of sql recording in the transaction sampler feature. Within the given block, no sql will be recorded
usage:
NewRelic::Agent.disable_sql_recording do
...
end
568 569 570 571 572 573 574 575 576 577 578 579 |
# File 'lib/new_relic/agent.rb', line 568 def disable_sql_recording record_api_supportability_metric(:disable_sql_recording) return yield unless agent state = agent.set_record_sql(false) begin yield ensure agent.set_record_sql(state) end end |
#drop_buffered_data ⇒ Object
Clear out any data the agent has buffered but has not yet transmitted to the collector.
453 454 455 456 457 |
# File 'lib/new_relic/agent.rb', line 453 def drop_buffered_data # the following line needs else branch coverage agent.drop_buffered_data if agent # rubocop:disable Style/SafeNavigation record_api_supportability_metric(:drop_buffered_data) end |
#get_transaction_name ⇒ Object
Get the name of the current running transaction. This is useful if you want to modify the default name.
711 712 713 714 715 716 717 718 719 |
# File 'lib/new_relic/agent.rb', line 711 def get_transaction_name # THREAD_LOCAL_ACCESS record_api_supportability_metric(:get_transaction_name) txn = Transaction.tl_current if txn namer = Instrumentation::ControllerInstrumentation::TransactionNamer txn.best_name.sub(Regexp.new("\\A#{Regexp.escape(namer.prefix_for_category(txn))}"), '') end end |
#ignore_apdex ⇒ Object
This method disables the recording of Apdex metrics in the current transaction.
522 523 524 525 |
# File 'lib/new_relic/agent.rb', line 522 def ignore_apdex record_api_supportability_metric(:ignore_apdex) NewRelic::Agent::Transaction.tl_current&.ignore_apdex! end |
#ignore_enduser ⇒ Object
This method disables browser monitoring javascript injection in the current transaction.
532 533 534 535 |
# File 'lib/new_relic/agent.rb', line 532 def ignore_enduser record_api_supportability_metric(:ignore_enduser) NewRelic::Agent::Transaction.tl_current&.ignore_enduser! end |
#ignore_error_filter(&block) ⇒ Object
Set a filter to be applied to errors that the Ruby Agent will track. The block should evaluate to the exception to track (which could be different from the original exception) or nil to ignore this exception.
The block is yielded to with the exception to filter.
Return the new block or the existing filter Proc if no block is passed.
254 255 256 257 258 259 260 261 262 |
# File 'lib/new_relic/agent.rb', line 254 def ignore_error_filter(&block) record_api_supportability_metric(:ignore_error_filter) if block NewRelic::Agent::ErrorCollector.ignore_error_filter = block else NewRelic::Agent::ErrorCollector.ignore_error_filter end end |
#ignore_transaction ⇒ Object
This method disables the recording of the current transaction. No metrics, traced errors, transaction traces, Insights events, slow SQL traces, or RUM injection will happen for this transaction.
512 513 514 515 |
# File 'lib/new_relic/agent.rb', line 512 def ignore_transaction record_api_supportability_metric(:ignore_transaction) NewRelic::Agent::Transaction.tl_current&.ignore! end |
#increment_metric(metric_name, amount = 1) ⇒ Object
Increment a simple counter metric.
metric_name should follow a slash separated path convention. Application specific metrics should begin with “Custom/”.
This method is safe to use from any thread.
227 228 229 230 231 232 233 234 235 236 237 |
# File 'lib/new_relic/agent.rb', line 227 def increment_metric(metric_name, amount = 1) # THREAD_LOCAL_ACCESS return unless agent if amount == 1 metrics = [metric_name, SUPPORTABILITY_INCREMENT_METRIC] agent.stats_engine.tl_record_unscoped_metrics(metrics) { |stats| stats.increment_count } else agent.stats_engine.tl_record_unscoped_metrics(metric_name) { |stats| stats.increment_count(amount) } agent.stats_engine.tl_record_unscoped_metrics(SUPPORTABILITY_INCREMENT_METRIC) { |stats| stats.increment_count } end end |
#linking_metadata ⇒ Object
Returns a new hash containing trace and entity metadata that can be used to relate data to a trace or to an entity in APM.
This hash includes:
-
trace.id - The current trace id, if there is a current trace id. This value may be omitted.
-
span.id - The current span id, if there is a current span. This value may be omitted.
-
entity.name - The name of the current application. This is read from the
app_namekey in your config. If there are multiple application names, the first one is used. -
entity.type - The entity type is hardcoded to the string ‘SERVICE’.
-
entity.guid - The guid of the current entity.
-
hostname - The fully qualified hostname.
781 782 783 784 785 786 |
# File 'lib/new_relic/agent.rb', line 781 def = Hash.new LinkingMetadata.() LinkingMetadata.() end |
#manual_start(options = {}) ⇒ Object
Call this to manually start the Agent in situations where the Agent does not auto-start.
When the app environment loads, so does the Agent. However, the Agent will only connect to the service if a web front-end is found. If you want to selectively monitor ruby processes that don’t use web plugins, then call this method in your code and the Agent will fire up and start reporting to the service.
Options are passed in as overrides for values in the newrelic.yml, such as app_name. In addition, the option log will take a logger that will be used instead of the standard file logger. The setting for the newrelic.yml section to use (ie, RAILS_ENV) can be overridden with an :env argument.
399 400 401 402 403 404 |
# File 'lib/new_relic/agent.rb', line 399 def manual_start( = {}) raise 'Options must be a hash' unless Hash === NewRelic::Control.instance.init_plugin({:agent_enabled => true, :sync_startup => true}.merge()) record_api_supportability_metric(:manual_start) end |
#notice_error(exception, options = {}) ⇒ Object
Send an error to New Relic.
Any option keys other than the ones listed here are treated as :custom_params.
Note: Previous versions of the agent allowed passing :request_params, but those are now ignored. If you need to record the request parameters, call this method inside a transaction or pass the information in :custom_params.
Most of the time, you do not need to specify the :uri or :metric options; only pass them if you are calling notice_error outside a transaction.
292 293 294 295 296 297 |
# File 'lib/new_relic/agent.rb', line 292 def notice_error(exception, = {}) record_api_supportability_metric(:notice_error) Transaction.notice_error(exception, ) nil # don't return a noticed error data structure. it can only hurt. end |
#record_custom_event(event_type, event_attrs) ⇒ Object
Record a custom event to be sent to New Relic Insights. The recorded event will be buffered in memory until the next time the agent sends data to New Relic’s servers.
If you want to be able to tie the information recorded via this call back to the web request or background job that it happened in, you may want to instead use the add_custom_attributes API call to attach attributes to the Transaction event that will automatically be generated for the request.
A timestamp will be automatically added to the recorded event when this method is called.
368 369 370 371 372 373 374 375 376 |
# File 'lib/new_relic/agent.rb', line 368 def record_custom_event(event_type, event_attrs) record_api_supportability_metric(:record_custom_event) if agent && NewRelic::Agent.config[:'custom_insights_events.enabled'] agent.custom_event_aggregator.record(event_type, event_attrs) end nil end |
#record_metric(metric_name, value) ⇒ Object
Record a value for the given metric name.
This method should be used to record event-based metrics such as method calls that are associated with a specific duration or magnitude.
metric_name should follow a slash separated path convention. Application specific metrics should begin with “Custom/”.
value should be either a single Numeric value representing the duration/ magnitude of the event being recorded, or a Hash containing :count, :total, :min, :max, and :sum_of_squares keys. The latter form is useful for recording pre-aggregated metrics collected externally.
This method is safe to use from any thread.
197 198 199 200 201 202 203 204 205 206 207 208 |
# File 'lib/new_relic/agent.rb', line 197 def record_metric(metric_name, value) # THREAD_LOCAL_ACCESS record_api_supportability_metric(:record_metric) return unless agent if value.is_a?(Hash) stats = NewRelic::Agent::Stats.new value = stats.hash_merge(value) end agent.stats_engine.tl_record_unscoped_metrics(metric_name, value) end |
#require_test_helper ⇒ Object
Require agent testing helper methods
476 477 478 479 |
# File 'lib/new_relic/agent.rb', line 476 def require_test_helper record_api_supportability_metric(:require_test_helper) require File.('../../../test/agent_helper', __FILE__) end |
#set_error_group_callback(callback_proc) ⇒ Object
Set a callback proc for determining an error’s error group name
Typically this method should be called only once to set a callback for use with all noticed errors. If it is called multiple times, each new callback given will replace the old one.
The proc will be called with a single hash as its input argument and is expected to return a string representing the desired error group.
see docs.newrelic.com/docs/errors-inbox/errors-inbox/#groups
The hash passed to the customer defined callback proc has the following keys:
:error => The Ruby error class instance, likely inheriting from
StandardError. Call `#class`, `#message`, and `#backtrace` on
the error object to retrieve the error's class, message, and
backtrace.
:customAttributes => Any customer defined custom attributes that have been
associated with the current transaction.
:‘request.uri’ => The current request URI if available :‘http.statusCode’ => The HTTP status code (200, 404, etc.) if available :‘http.method’ => The HTTP method (GET, PUT, etc.) if available :‘error.expected’ => Whether (true) or not (false) the error was expected :options => The options hash passed to ‘NewRelic::Agent.notice_error`
329 330 331 332 333 334 335 336 337 338 |
# File 'lib/new_relic/agent.rb', line 329 def set_error_group_callback(callback_proc) unless callback_proc.is_a?(Proc) NewRelic::Agent.logger.error("#{self}.#{__method__}: expected an argument of type Proc, " \ "got #{callback_proc.class}") return end record_api_supportability_metric(:set_error_group_callback) @error_group_callback = callback_proc end |
#set_sql_obfuscator(type = :replace, &block) ⇒ Object
This method sets the block sent to this method as a sql obfuscator. The block will be called with a single String SQL statement to obfuscate. The method must return the obfuscated String SQL. If chaining of obfuscators is required, use type = :before or :after
type = :before, :replace, :after
Example:
NewRelic::Agent.set_sql_obfuscator(:replace) do |sql|
my_obfuscator(sql)
end
497 498 499 500 |
# File 'lib/new_relic/agent.rb', line 497 def set_sql_obfuscator(type = :replace, &block) record_api_supportability_metric(:set_sql_obfuscator) NewRelic::Agent::Database.set_sql_obfuscator(type, &block) end |
#set_transaction_name(name, options = {}) ⇒ Object
Set the name of the current running transaction. The agent will apply a reasonable default based on framework routing, but in cases where this is insufficient, this can be used to manually control the name of the transaction.
The category of transaction can be specified via the :category option. The following are the only valid categories:
-
:category => :controllerindicates that this is a controller action and will appear with all the other actions. -
:category => :taskindicates that this is a background task and will show up in New Relic with other background tasks instead of in the controllers list -
:category => :middlewareif you are instrumenting a rack middleware call. The:nameis optional, useful if you have more than one potential transaction in the #call. -
:category => :uriindicates that this is a web transaction whose name is a normalized URI, where ‘normalized’ means the URI does not have any elements with data in them such as in many REST URIs.
The default category is the same as the running transaction.
701 702 703 704 |
# File 'lib/new_relic/agent.rb', line 701 def set_transaction_name(name, = {}) record_api_supportability_metric(:set_transaction_name) Transaction.set_overriding_transaction_name(name, [:category]) end |
#set_user_id(user_id) ⇒ Object
Set the user id for the current transaction. When present, this value will be included in the agent attributes for transaction and error events as ‘enduser.id’.
657 658 659 660 661 662 663 664 665 666 667 668 669 670 |
# File 'lib/new_relic/agent.rb', line 657 def set_user_id(user_id) record_api_supportability_metric(:set_user_id) if user_id.nil? || user_id.empty? ::NewRelic::Agent.logger.warn('NewRelic::Agent.set_user_id called with a nil or empty user id.') return end default_destinations = NewRelic::Agent::AttributeFilter::DST_TRANSACTION_TRACER | NewRelic::Agent::AttributeFilter::DST_TRANSACTION_EVENTS | NewRelic::Agent::AttributeFilter::DST_ERROR_COLLECTOR NewRelic::Agent::Transaction.add_agent_attribute(:'enduser.id', user_id, default_destinations) end |
#shutdown(options = {}) ⇒ Object
Shutdown the agent. Call this before exiting. Sends any queued data and kills the background thread.
444 445 446 447 |
# File 'lib/new_relic/agent.rb', line 444 def shutdown( = {}) record_api_supportability_metric(:shutdown) agent&.shutdown end |