Class: NewRelic::Agent::Configuration::DefaultSource
- Inherits:
-
Object
- Object
- NewRelic::Agent::Configuration::DefaultSource
- Extended by:
- Forwardable
- Defined in:
- lib/new_relic/agent/configuration/default_source.rb
Constant Summary collapse
- DEFAULT_LOG_DIR =
'log/'.freeze
- SEMICOLON =
';'.freeze
Instance Attribute Summary collapse
- #defaults ⇒ Object readonly
Class Method Summary collapse
- .agent_enabled ⇒ Object
- .allowlist_for(key) ⇒ Object
- .api_host ⇒ Object
- .app_name ⇒ Object
- .audit_log_path ⇒ Object
- .config_path ⇒ Object
- .config_search_paths ⇒ Object
- .convert_to_constant_list(raw_value) ⇒ Object
- .convert_to_hash(value) ⇒ Object
- .convert_to_list(value) ⇒ Object
- .convert_to_list_on_semicolon(value) ⇒ Object
- .convert_to_regexp_list(raw_value) ⇒ Object
- .default_for(key) ⇒ Object
- .default_settings(key) ⇒ Object
- .dispatcher ⇒ Object
- .enforce_fallback(allowed_values: nil, fallback: nil) ⇒ Object
- .framework ⇒ Object
- .host ⇒ Object
- .profiling_available ⇒ Object
- .thread_profiler_enabled ⇒ Object
- .transaction_tracer_transaction_threshold ⇒ Object
- .transform_for(key) ⇒ Object
- .value_from_defaults(key, subkey) ⇒ Object
Instance Method Summary collapse
- #default_values ⇒ Object
-
#initialize ⇒ DefaultSource
constructor
A new instance of DefaultSource.
Constructor Details
#initialize ⇒ DefaultSource
Returns a new instance of DefaultSource.
43 44 45 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 43 def initialize @defaults = default_values end |
Instance Attribute Details
#defaults ⇒ Object (readonly)
38 39 40 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 38 def defaults @defaults end |
Class Method Details
.agent_enabled ⇒ Object
143 144 145 146 147 148 149 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 143 def self.agent_enabled proc { NewRelic::Agent.config[:enabled] && (NewRelic::Agent.config[:test_mode] || NewRelic::Agent.config[:monitor_mode]) && NewRelic::Agent::Autostart.agent_should_start? } end |
.allowlist_for(key) ⇒ Object
63 64 65 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 63 def self.allowlist_for(key) value_from_defaults(key, :allowlist) end |
.api_host ⇒ Object
201 202 203 204 205 206 207 208 209 210 211 212 213 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 201 def self.api_host # only used for deployment task proc do api_version = if NewRelic::Agent.config[:api_key].nil? || NewRelic::Agent.config[:api_key].empty? 'rpm' else 'api' end api_region = 'eu.' if String(NewRelic::Agent.config[:license_key]).start_with?('eu') "#{api_version}.#{api_region}newrelic.com" end end |
.app_name ⇒ Object
163 164 165 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 163 def self.app_name proc { NewRelic::Control.instance.env } end |
.audit_log_path ⇒ Object
153 154 155 156 157 158 159 160 161 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 153 def self.audit_log_path proc { log_file_path = NewRelic::Agent.config[:log_file_path] wants_stdout = (log_file_path.casecmp(NewRelic::STANDARD_OUT) == 0) audit_log_dir = wants_stdout ? DEFAULT_LOG_DIR : log_file_path File.join(audit_log_dir, 'newrelic_audit.log') } end |
.config_path ⇒ Object
110 111 112 113 114 115 116 117 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 110 def self.config_path proc { found_path = NewRelic::Agent.config[:config_search_paths].detect do |file| File.(file) if File.exist?(file) end found_path || NewRelic::EMPTY_STR } end |
.config_search_paths ⇒ Object
75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 75 def self.config_search_paths proc { yaml = 'newrelic.yml' config_yaml = File.join('config', yaml) erb = 'newrelic.yml.erb' config_erb = File.join('config', erb) paths = [config_yaml, yaml, config_erb, erb] if NewRelic::Control.instance.root paths << File.join(NewRelic::Control.instance.root, config_yaml) paths << File.join(NewRelic::Control.instance.root, yaml) paths << File.join(NewRelic::Control.instance.root, config_erb) paths << File.join(NewRelic::Control.instance.root, erb) end if ENV['HOME'] paths << File.join(ENV['HOME'], '.newrelic', yaml) paths << File.join(ENV['HOME'], yaml) paths << File.join(ENV['HOME'], '.newrelic', erb) paths << File.join(ENV['HOME'], erb) end # If we're packaged for warbler, we can tell from GEM_HOME # the following line needs else branch coverage if ENV['GEM_HOME'] && ENV['GEM_HOME'].end_with?('.jar!') # rubocop:disable Style/SafeNavigation app_name = File.basename(ENV['GEM_HOME'], '.jar!') paths << File.join(ENV['GEM_HOME'], app_name, config_yaml) paths << File.join(ENV['GEM_HOME'], app_name, config_erb) end paths } end |
.convert_to_constant_list(raw_value) ⇒ Object
258 259 260 261 262 263 264 265 266 267 268 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 258 def self.convert_to_constant_list(raw_value) return NewRelic::EMPTY_ARRAY if raw_value.nil? || raw_value.empty? constants = convert_to_list(raw_value).map! do |class_name| const = ::NewRelic::LanguageSupport.constantize(class_name) NewRelic::Agent.logger.warn("Ignoring invalid constant '#{class_name}' in #{raw_value}") unless const const end constants.compact! constants end |
.convert_to_hash(value) ⇒ Object
233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 233 def self.convert_to_hash(value) return value if value.is_a?(Hash) if value.is_a?(String) return value.split(',').each_with_object({}) do |item, hash| key, value = item.split('=') hash[key] = value end end raise ArgumentError.new( "Config value '#{value}' of " \ "class #{value.class} couldn't be turned into a Hash." ) end |
.convert_to_list(value) ⇒ Object
222 223 224 225 226 227 228 229 230 231 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 222 def self.convert_to_list(value) case value when String value.split(/\s*,\s*/) when Array value else raise ArgumentError.new("Config value '#{value}' couldn't be turned into a list.") end end |
.convert_to_list_on_semicolon(value) ⇒ Object
250 251 252 253 254 255 256 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 250 def self.convert_to_list_on_semicolon(value) case value when Array then value when String then value.split(SEMICOLON) else NewRelic::EMPTY_ARRAY end end |
.convert_to_regexp_list(raw_value) ⇒ Object
215 216 217 218 219 220 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 215 def self.convert_to_regexp_list(raw_value) value_list = convert_to_list(raw_value) value_list.map do |value| /#{value}/ end end |
.default_for(key) ⇒ Object
67 68 69 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 67 def self.default_for(key) value_from_defaults(key, :default) end |
.default_settings(key) ⇒ Object
55 56 57 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 55 def self.default_settings(key) ::NewRelic::Agent::Configuration::DEFAULTS[key] end |
.dispatcher ⇒ Object
167 168 169 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 167 def self.dispatcher proc { NewRelic::Control.instance.local_env.discovered_dispatcher } end |
.enforce_fallback(allowed_values: nil, fallback: nil) ⇒ Object
270 271 272 273 274 275 276 277 278 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 270 def self.enforce_fallback(allowed_values: nil, fallback: nil) proc do |configured_value| if allowed_values.any? { |v| v =~ /#{configured_value}/i } configured_value else fallback end end end |
.framework ⇒ Object
119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 119 def self.framework proc { case when defined?(::NewRelic::TEST) then :test when defined?(::Rails::VERSION) case Rails::VERSION::MAJOR when 3 :rails3 when 4..7 :rails_notifications else ::NewRelic::Agent.logger.warn("Detected untested Rails version #{Rails::VERSION::STRING}") :rails_notifications end when defined?(::Padrino) && defined?(::Padrino::PathRouter::Router) then :padrino when defined?(::Sinatra) && defined?(::Sinatra::Base) then :sinatra when defined?(::Roda) then :roda when defined?(::Grape) then :grape when defined?(::NewRelic::IA) then :external else :ruby end } end |
.host ⇒ Object
190 191 192 193 194 195 196 197 198 199 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 190 def self.host proc do regex = /\A(?<identifier>.+?)x/ if matches = regex.match(String(NewRelic::Agent.config[:license_key])) "collector.#{matches['identifier']}.nr-data.net" else 'collector.newrelic.com' end end end |
.profiling_available ⇒ Object
179 180 181 182 183 184 185 186 187 188 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 179 def self.profiling_available proc { begin require 'ruby-prof' true rescue LoadError false end } end |
.thread_profiler_enabled ⇒ Object
171 172 173 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 171 def self.thread_profiler_enabled proc { NewRelic::Agent::Threading::BacktraceService.is_supported? } end |
.transaction_tracer_transaction_threshold ⇒ Object
175 176 177 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 175 def self.transaction_tracer_transaction_threshold proc { NewRelic::Agent.config[:apdex_t] * 4 } end |
.transform_for(key) ⇒ Object
71 72 73 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 71 def self.transform_for(key) value_from_defaults(key, :transform) end |
.value_from_defaults(key, subkey) ⇒ Object
59 60 61 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 59 def self.value_from_defaults(key, subkey) default_settings(key)&.send(:[], subkey) end |
Instance Method Details
#default_values ⇒ Object
47 48 49 50 51 52 53 |
# File 'lib/new_relic/agent/configuration/default_source.rb', line 47 def default_values result = {} ::NewRelic::Agent::Configuration::DEFAULTS.each do |key, value| result[key] = value[:default] end result end |