Class: NewRelic::Agent::Transaction
- Inherits:
-
Object
- Object
- NewRelic::Agent::Transaction
show all
- Includes:
- Tracing
- Defined in:
- lib/new_relic/agent/transaction.rb,
lib/new_relic/agent/transaction/trace.rb,
lib/new_relic/agent/transaction/segment.rb,
lib/new_relic/agent/transaction/tracing.rb,
lib/new_relic/agent/transaction/trace_node.rb,
lib/new_relic/agent/transaction/trace_builder.rb,
lib/new_relic/agent/transaction/trace_context.rb,
lib/new_relic/agent/transaction/abstract_segment.rb,
lib/new_relic/agent/transaction/datastore_segment.rb,
lib/new_relic/agent/transaction/distributed_tracer.rb,
lib/new_relic/agent/transaction/request_attributes.rb,
lib/new_relic/agent/transaction/distributed_tracing.rb,
lib/new_relic/agent/transaction/slowest_sample_buffer.rb,
lib/new_relic/agent/transaction/message_broker_segment.rb,
lib/new_relic/agent/transaction/external_request_segment.rb,
lib/new_relic/agent/transaction/synthetics_sample_buffer.rb,
lib/new_relic/agent/transaction/transaction_sample_buffer.rb
Overview
This class represents a single transaction (usually mapping to one web request or background job invocation) instrumented by the Ruby agent.
Defined Under Namespace
Modules: DistributedTracing, TraceBuilder, TraceContext, Tracing
Classes: AbstractSegment, DatastoreSegment, DistributedTracer, ExternalRequestSegment, MessageBrokerSegment, RequestAttributes, Segment, SlowestSampleBuffer, SyntheticsSampleBuffer, Trace, TraceNode, TransactionSampleBuffer
Constant Summary
collapse
- NESTED_TRANSACTION_PREFIX =
'Nested/'
- CONTROLLER_PREFIX =
'Controller/'
- MIDDLEWARE_PREFIX =
'Middleware/Rack/'
- OTHER_TRANSACTION_PREFIX =
'OtherTransaction/'
- TASK_PREFIX =
"#{OTHER_TRANSACTION_PREFIX}Background/"
- RAKE_PREFIX =
"#{OTHER_TRANSACTION_PREFIX}Rake/"
- MESSAGE_PREFIX =
"#{OTHER_TRANSACTION_PREFIX}Message/"
- RACK_PREFIX =
"#{CONTROLLER_PREFIX}Rack/"
- SINATRA_PREFIX =
"#{CONTROLLER_PREFIX}Sinatra/"
- GRAPE_PREFIX =
"#{CONTROLLER_PREFIX}Grape/"
- ACTION_CABLE_PREFIX =
"#{CONTROLLER_PREFIX}ActionCable/"
- WEB_TRANSACTION_CATEGORIES =
[:web, :controller, :uri, :rack, :sinatra, :grape, :middleware, :action_cable].freeze
- MIDDLEWARE_SUMMARY_METRICS =
['Middleware/all'].freeze
- WEB_SUMMARY_METRIC =
'HttpDispatcher'
- OTHER_SUMMARY_METRIC =
"#{OTHER_TRANSACTION_PREFIX}all"
- QUEUE_TIME_METRIC =
'WebFrontend/QueueTime'
- APDEX_S =
'S'
- APDEX_T =
'T'
- APDEX_F =
'F'
- APDEX_ALL_METRIC =
'ApdexAll'
- APDEX_METRIC =
'Apdex'
- APDEX_OTHER_METRIC =
'ApdexOther'
- APDEX_TXN_METRIC_PREFIX =
'Apdex/'
- APDEX_OTHER_TXN_METRIC_PREFIX =
'ApdexOther/Transaction/'
- JRUBY_CPU_TIME_ERROR =
'Error calculating JRuby CPU Time'
- @@java_classes_loaded =
false
Instance Attribute Summary collapse
Attributes included from Tracing
#async, #current_segment_by_thread, #total_time
Class Method Summary
collapse
-
.abort_transaction! ⇒ Object
discards the currently saved transaction information.
-
.add_agent_attribute(key, value, default_destinations) ⇒ Object
-
.apdex_bucket(duration, failed, apdex_t) ⇒ Object
-
.merge_untrusted_agent_attributes(attributes, prefix, default_destinations) ⇒ Object
-
.name_from_partial(partial_name, category) ⇒ Object
-
.nested_transaction_name(name) ⇒ Object
-
.notice_error(e, options = {}) ⇒ Object
See NewRelic::Agent.notice_error for options and commentary.
-
.recording_web_transaction? ⇒ Boolean
Returns truthy if the current in-progress transaction is considered a a web transaction (as opposed to, e.g., a background transaction).
-
.set_default_transaction_name(partial_name, category = nil) ⇒ Object
-
.set_overriding_transaction_name(partial_name, category = nil) ⇒ Object
-
.start_new_transaction(state, category, options) ⇒ Object
-
.tl_current ⇒ Object
Return the currently active transaction, or nil.
Instance Method Summary
collapse
-
#abort_transaction! ⇒ Object
Call this to ensure that the current transaction trace is not saved To fully ignore all metrics and errors, use ignore! instead.
-
#add_agent_attribute(key, value, default_destinations) ⇒ Object
-
#add_custom_attributes(p) ⇒ Object
-
#add_log_event(event) ⇒ Object
-
#apdex_bucket(duration, current_apdex_t) ⇒ Object
-
#apdex_t ⇒ Object
-
#append_apdex_perf_zone(payload) ⇒ Object
-
#append_synthetics_to(payload) ⇒ Object
-
#assign_agent_attributes ⇒ Object
-
#assign_intrinsics ⇒ Object
-
#assign_segment_dt_attributes ⇒ Object
-
#background_summary_metrics ⇒ Object
-
#best_name ⇒ Object
(also: #transaction_name)
-
#calculate_gc_time ⇒ Object
-
#calculate_transport_duration(distributed_trace_payload) ⇒ Object
This method returns transport_duration in seconds.
-
#commit!(outermost_node_name) ⇒ Object
-
#cpu_burn ⇒ Object
-
#create_initial_segment(options = {}) ⇒ Object
-
#create_nested_segment(category, options) ⇒ Object
-
#create_segment(name, options = {}) ⇒ Object
-
#current_segment ⇒ Object
-
#current_segment_key ⇒ Object
-
#default_name=(name) ⇒ Object
-
#distributed_tracer ⇒ Object
-
#finish ⇒ Object
-
#finished? ⇒ Boolean
-
#freeze_name_and_execute_if_not_ignored ⇒ Object
-
#generate_payload ⇒ Object
-
#had_error_affecting_apdex? ⇒ Boolean
-
#ignore! ⇒ Object
-
#ignore? ⇒ Boolean
-
#ignore_apdex! ⇒ Object
-
#ignore_apdex? ⇒ Boolean
-
#ignore_enduser! ⇒ Object
-
#ignore_enduser? ⇒ Boolean
-
#ignore_trace? ⇒ Boolean
-
#include_guid? ⇒ Boolean
-
#influences_transaction_name?(category) ⇒ Boolean
-
#initial_segment ⇒ Object
-
#initialize(category, options) ⇒ Transaction
constructor
rubocop:disable Metrics/AbcSize.
-
#instrumentation_state ⇒ Object
This transaction-local hash may be used as temporary storage by instrumentation that needs to pass data from one instrumentation point to another.
-
#is_synthetics_request? ⇒ Boolean
-
#jruby_cpu_burn ⇒ Object
-
#log_frozen_name(name) ⇒ Object
-
#merge_metrics ⇒ Object
-
#merge_request_parameters(params) ⇒ Object
-
#merge_untrusted_agent_attributes(attributes, prefix, default_destinations) ⇒ Object
-
#name_frozen? ⇒ Boolean
-
#needs_middleware_summary_metrics?(name) ⇒ Boolean
-
#nest_initial_segment ⇒ Object
-
#normal_cpu_burn ⇒ Object
-
#notice_error(error, options = {}) ⇒ Object
-
#overridden_name=(name) ⇒ Object
-
#parent_segment_key ⇒ Object
-
#promoted_transaction_name(name) ⇒ Object
-
#queue_time ⇒ Object
-
#record_apdex ⇒ Object
-
#record_apdex_metrics(rollup_metric, transaction_prefix, current_apdex_t) ⇒ Object
-
#record_exception(exception, options, error_recorded) ⇒ Object
-
#record_exceptions ⇒ Object
-
#record_log_events ⇒ Object
-
#record_queue_time ⇒ Object
-
#record_summary_metrics(outermost_node_name) ⇒ Object
The summary metrics recorded by this method all end up with a duration equal to the transaction itself, and an exclusive time of zero.
-
#record_transaction_event ⇒ Object
-
#recording_web_transaction? ⇒ Boolean
-
#referer ⇒ Object
-
#remove_current_segment_by_thread_id(id) ⇒ Object
-
#request_path ⇒ Object
-
#request_port ⇒ Object
-
#sampled? ⇒ Boolean
-
#send_transaction_finished_event ⇒ Object
This event is fired when the transaction is fully completed.
-
#set_current_segment(new_segment) ⇒ Object
-
#set_default_transaction_name(name, category) ⇒ Object
-
#set_overriding_transaction_name(name, category) ⇒ Object
-
#similar_category?(category) ⇒ Boolean
-
#start(options = {}) ⇒ Object
-
#state ⇒ Object
-
#summary_metrics ⇒ Object
-
#synthetics_account_id ⇒ Object
-
#synthetics_job_id ⇒ Object
-
#synthetics_monitor_id ⇒ Object
-
#synthetics_resource_id ⇒ Object
-
#synthetics_version ⇒ Object
-
#threshold ⇒ Object
-
#trace_id ⇒ Object
-
#trace_id=(value) ⇒ Object
-
#transaction_specific_apdex_t ⇒ Object
-
#user_defined_rules_ignore? ⇒ Boolean
-
#web_category?(category) ⇒ Boolean
-
#with_database_metric_name(model, method, product = nil) ⇒ Object
Methods included from Tracing
#add_segment, #async?, #segment_complete, #segment_limit
Constructor Details
#initialize(category, options) ⇒ Transaction
rubocop:disable Metrics/AbcSize
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
|
# File 'lib/new_relic/agent/transaction.rb', line 213
def initialize(category, options) @nesting_max_depth = 0
@current_segment_by_thread = {}
@current_segment_lock = Mutex.new
@segments = []
self.default_name = options[:transaction_name]
@overridden_name = nil
@frozen_name = nil
@category = category
@start_time = Process.clock_gettime(Process::CLOCK_REALTIME)
@end_time = nil
@duration = nil
@apdex_start = options[:apdex_start_time] || @start_time
@jruby_cpu_start = jruby_cpu_time
@process_cpu_start = process_cpu
@gc_start_snapshot = NewRelic::Agent::StatsEngine::GCProfiler.take_snapshot
@filtered_params = options[:filtered_params] || {}
@exceptions = {}
@metrics = TransactionMetrics.new
@logs = PrioritySampledBuffer.new(NewRelic::Agent.instance.log_event_aggregator.capacity)
@guid = NewRelic::Agent::GuidGenerator.generate_guid
@ignore_this_transaction = false
@ignore_apdex = options.fetch(:ignore_apdex, false)
@ignore_enduser = options.fetch(:ignore_enduser, false)
@ignore_trace = false
@sampled = nil
@priority = nil
@starting_thread_id = Thread.current.object_id
@starting_segment_key = current_segment_key
@attributes = Attributes.new(NewRelic::Agent.instance.attribute_filter)
merge_request_parameters(@filtered_params)
if request = options[:request]
@request_attributes = RequestAttributes.new(request)
else
@request_attributes = nil
end
end
|
Instance Attribute Details
#apdex_start ⇒ Object
A Time instance used for calculating the apdex score, which might end up being @start, or it might be further upstream if we can find a request header for the queue entry time
62
63
64
|
# File 'lib/new_relic/agent/transaction.rb', line 62
def apdex_start
@apdex_start
end
|
#attributes ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def attributes
@attributes
end
|
#category ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def category
@category
end
|
#duration ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def duration
@duration
end
|
#end_time ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def end_time
@end_time
end
|
#exceptions ⇒ Object
64
65
66
|
# File 'lib/new_relic/agent/transaction.rb', line 64
def exceptions
@exceptions
end
|
#filtered_params ⇒ Object
64
65
66
|
# File 'lib/new_relic/agent/transaction.rb', line 64
def filtered_params
@filtered_params
end
|
#gc_start_snapshot ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def gc_start_snapshot
@gc_start_snapshot
end
|
#guid ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def guid
@guid
end
|
#http_response_code ⇒ Object
64
65
66
|
# File 'lib/new_relic/agent/transaction.rb', line 64
def http_response_code
@http_response_code
end
|
#jruby_cpu_start ⇒ Object
64
65
66
|
# File 'lib/new_relic/agent/transaction.rb', line 64
def jruby_cpu_start
@jruby_cpu_start
end
|
#logs ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def logs
@logs
end
|
#metrics ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def metrics
@metrics
end
|
#nesting_max_depth ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def nesting_max_depth
@nesting_max_depth
end
|
#parent_span_id ⇒ Object
64
65
66
|
# File 'lib/new_relic/agent/transaction.rb', line 64
def parent_span_id
@parent_span_id
end
|
#payload ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def payload
@payload
end
|
#priority ⇒ Object
308
309
310
|
# File 'lib/new_relic/agent/transaction.rb', line 308
def priority
@priority ||= (sampled? ? rand + 1.0 : rand).round(NewRelic::PRIORITY_PRECISION)
end
|
#process_cpu_start ⇒ Object
64
65
66
|
# File 'lib/new_relic/agent/transaction.rb', line 64
def process_cpu_start
@process_cpu_start
end
|
Fields for tracking synthetics requests
92
93
94
|
# File 'lib/new_relic/agent/transaction.rb', line 92
def
@raw_synthetics_header
end
|
#response_content_length ⇒ Object
64
65
66
|
# File 'lib/new_relic/agent/transaction.rb', line 64
def response_content_length
@response_content_length
end
|
#response_content_type ⇒ Object
64
65
66
|
# File 'lib/new_relic/agent/transaction.rb', line 64
def response_content_type
@response_content_type
end
|
#sampled=(value) ⇒ Object
85
86
87
|
# File 'lib/new_relic/agent/transaction.rb', line 85
def sampled=(value)
@sampled = value
end
|
#segments ⇒ Object
73
74
75
|
# File 'lib/new_relic/agent/transaction.rb', line 73
def segments
@segments
end
|
#start_time ⇒ Object
A Time instance for the start time, never nil
57
58
59
|
# File 'lib/new_relic/agent/transaction.rb', line 57
def start_time
@start_time
end
|
#synthetics_payload ⇒ Object
Fields for tracking synthetics requests
92
93
94
|
# File 'lib/new_relic/agent/transaction.rb', line 92
def synthetics_payload
@synthetics_payload
end
|
#transaction_trace ⇒ Object
Populated with the trace sample once this transaction is completed.
89
90
91
|
# File 'lib/new_relic/agent/transaction.rb', line 89
def transaction_trace
@transaction_trace
end
|
Class Method Details
.abort_transaction! ⇒ Object
discards the currently saved transaction information
134
135
136
137
138
|
# File 'lib/new_relic/agent/transaction.rb', line 134
def self.abort_transaction!
if txn = Tracer.current_transaction
txn.abort_transaction!
end
end
|
.add_agent_attribute(key, value, default_destinations) ⇒ Object
174
175
176
177
178
179
180
|
# File 'lib/new_relic/agent/transaction.rb', line 174
def self.add_agent_attribute(key, value, default_destinations)
if txn = tl_current
txn.add_agent_attribute(key, value, default_destinations)
else
NewRelic::Agent.logger.debug("Attempted to add agent attribute: #{key} without transaction")
end
end
|
.apdex_bucket(duration, failed, apdex_t) ⇒ Object
161
162
163
164
165
166
167
168
169
170
171
172
|
# File 'lib/new_relic/agent/transaction.rb', line 161
def self.apdex_bucket(duration, failed, apdex_t)
case
when failed
:apdex_f
when duration <= apdex_t
:apdex_s
when duration <= apdex_t * 4
:apdex_t
else
:apdex_f
end
end
|
.merge_untrusted_agent_attributes(attributes, prefix, default_destinations) ⇒ Object
188
189
190
191
192
193
194
|
# File 'lib/new_relic/agent/transaction.rb', line 188
def self.merge_untrusted_agent_attributes(attributes, prefix, default_destinations)
if txn = tl_current
txn.merge_untrusted_agent_attributes(attributes, prefix, default_destinations)
else
NewRelic::Agent.logger.debug('Attempted to merge untrusted attributes without transaction')
end
end
|
.name_from_partial(partial_name, category) ⇒ Object
.nested_transaction_name(name) ⇒ Object
.notice_error(e, options = {}) ⇒ Object
See NewRelic::Agent.notice_error for options and commentary
141
142
143
144
145
146
147
|
# File 'lib/new_relic/agent/transaction.rb', line 141
def self.notice_error(e, options = {})
if txn = Tracer.current_transaction
txn.notice_error(e, options)
elsif NewRelic::Agent.instance
NewRelic::Agent.instance.error_collector.notice_error(e, options)
end
end
|
.recording_web_transaction? ⇒ Boolean
Returns truthy if the current in-progress transaction is considered a a web transaction (as opposed to, e.g., a background transaction).
154
155
156
157
158
159
|
# File 'lib/new_relic/agent/transaction.rb', line 154
def self.recording_web_transaction? NewRelic::Agent.record_api_supportability_metric(:recording_web_transaction?)
txn = tl_current
txn&.recording_web_transaction?
end
|
.set_default_transaction_name(partial_name, category = nil) ⇒ Object
99
100
101
102
103
|
# File 'lib/new_relic/agent/transaction.rb', line 99
def self.set_default_transaction_name(partial_name, category = nil) txn = tl_current
name = name_from_partial(partial_name, category || txn.category)
txn.set_default_transaction_name(name, category)
end
|
.set_overriding_transaction_name(partial_name, category = nil) ⇒ Object
105
106
107
108
109
110
111
|
# File 'lib/new_relic/agent/transaction.rb', line 105
def self.set_overriding_transaction_name(partial_name, category = nil) txn = tl_current
return unless txn
name = name_from_partial(partial_name, category || txn.category)
txn.set_overriding_transaction_name(name, category)
end
|
.start_new_transaction(state, category, options) ⇒ Object
118
119
120
121
122
123
|
# File 'lib/new_relic/agent/transaction.rb', line 118
def self.start_new_transaction(state, category, options)
txn = Transaction.new(category, options)
state.reset(txn)
txn.start(options)
txn
end
|
.tl_current ⇒ Object
Return the currently active transaction, or nil.
Instance Method Details
#abort_transaction! ⇒ Object
Call this to ensure that the current transaction trace is not saved To fully ignore all metrics and errors, use ignore! instead.
496
497
498
|
# File 'lib/new_relic/agent/transaction.rb', line 496
def abort_transaction!
@ignore_trace = true
end
|
#add_agent_attribute(key, value, default_destinations) ⇒ Object
182
183
184
185
186
|
# File 'lib/new_relic/agent/transaction.rb', line 182
def add_agent_attribute(key, value, default_destinations)
@attributes.add_agent_attribute(key, value, default_destinations)
current_segment.add_agent_attribute(key, value) if current_segment end
|
#add_custom_attributes(p) ⇒ Object
869
870
871
|
# File 'lib/new_relic/agent/transaction.rb', line 869
def add_custom_attributes(p)
attributes.merge_custom_attributes(p)
end
|
#add_log_event(event) ⇒ Object
873
874
875
|
# File 'lib/new_relic/agent/transaction.rb', line 873
def add_log_event(event)
logs.append(event: event)
end
|
#apdex_bucket(duration, current_apdex_t) ⇒ Object
804
805
806
|
# File 'lib/new_relic/agent/transaction.rb', line 804
def apdex_bucket(duration, current_apdex_t)
self.class.apdex_bucket(duration, had_error_affecting_apdex?, current_apdex_t)
end
|
#apdex_t ⇒ Object
835
836
837
|
# File 'lib/new_relic/agent/transaction.rb', line 835
def apdex_t
transaction_specific_apdex_t || Agent.config[:apdex_t]
end
|
#append_apdex_perf_zone(payload) ⇒ Object
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
|
# File 'lib/new_relic/agent/transaction.rb', line 709
def append_apdex_perf_zone(payload)
if recording_web_transaction?
bucket = apdex_bucket(duration, apdex_t)
elsif background_apdex_t = transaction_specific_apdex_t
bucket = apdex_bucket(duration, background_apdex_t)
end
return unless bucket
bucket_str = case bucket
when :apdex_s then APDEX_S
when :apdex_t then APDEX_T
when :apdex_f then APDEX_F
end
payload[:apdex_perf_zone] = bucket_str if bucket_str
end
|
#append_synthetics_to(payload) ⇒ Object
726
727
728
729
730
731
732
|
# File 'lib/new_relic/agent/transaction.rb', line 726
def append_synthetics_to(payload)
return unless is_synthetics_request?
payload[:synthetics_resource_id] = synthetics_resource_id
payload[:synthetics_job_id] = synthetics_job_id
payload[:synthetics_monitor_id] = synthetics_monitor_id
end
|
#assign_agent_attributes ⇒ Object
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
|
# File 'lib/new_relic/agent/transaction.rb', line 585
def assign_agent_attributes
default_destinations = AttributeFilter::DST_TRANSACTION_TRACER |
AttributeFilter::DST_TRANSACTION_EVENTS |
AttributeFilter::DST_ERROR_COLLECTOR
if http_response_code
add_agent_attribute(:'http.statusCode', http_response_code, default_destinations)
end
if response_content_length
add_agent_attribute(:'response.headers.contentLength', response_content_length.to_i, default_destinations)
end
if response_content_type
add_agent_attribute(:'response.headers.contentType', response_content_type, default_destinations)
end
@request_attributes&.assign_agent_attributes(self)
display_host = Agent.config[:'process_host.display_name']
unless display_host == NewRelic::Agent::Hostname.get
add_agent_attribute(:'host.displayName', display_host, default_destinations)
end
end
|
#assign_intrinsics ⇒ Object
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
|
# File 'lib/new_relic/agent/transaction.rb', line 610
def assign_intrinsics
attributes.add_intrinsic_attribute(:priority, priority)
if gc_time = calculate_gc_time
attributes.add_intrinsic_attribute(:gc_time, gc_time)
end
if burn = cpu_burn
attributes.add_intrinsic_attribute(:cpu_time, burn)
end
if is_synthetics_request?
attributes.add_intrinsic_attribute(:synthetics_resource_id, synthetics_resource_id)
attributes.add_intrinsic_attribute(:synthetics_job_id, synthetics_job_id)
attributes.add_intrinsic_attribute(:synthetics_monitor_id, synthetics_monitor_id)
end
distributed_tracer.assign_intrinsics
end
|
#assign_segment_dt_attributes ⇒ Object
578
579
580
581
582
583
|
# File 'lib/new_relic/agent/transaction.rb', line 578
def assign_segment_dt_attributes
dt_payload = distributed_tracer.trace_state_payload || distributed_tracer.distributed_trace_payload
parent_attributes = {}
DistributedTraceAttributes.copy_parent_attributes(self, dt_payload, parent_attributes)
parent_attributes.each { |k, v| initial_segment.add_agent_attribute(k, v) }
end
|
#background_summary_metrics ⇒ Object
508
509
510
511
512
513
514
515
|
# File 'lib/new_relic/agent/transaction.rb', line 508
def background_summary_metrics
segments = @frozen_name.split('/')
if segments.size > 2
["OtherTransaction/#{segments[1]}/all", OTHER_SUMMARY_METRIC]
else
[]
end
end
|
#best_name ⇒ Object
Also known as:
transaction_name
379
380
381
382
383
384
|
# File 'lib/new_relic/agent/transaction.rb', line 379
def best_name
@frozen_name ||
@overridden_name ||
@default_name ||
NewRelic::Agent::UNKNOWN_METRIC
end
|
#calculate_gc_time ⇒ Object
#calculate_transport_duration(distributed_trace_payload) ⇒ Object
This method returns transport_duration in seconds. Transport duration is stored in milliseconds on the payload, but it’s needed in seconds for metrics and intrinsics.
638
639
640
641
642
643
|
# File 'lib/new_relic/agent/transaction.rb', line 638
def calculate_transport_duration(distributed_trace_payload)
return unless distributed_trace_payload
duration = start_time - (distributed_trace_payload.timestamp / 1000.0)
[duration, 0].max
end
|
#commit!(outermost_node_name) ⇒ Object
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
|
# File 'lib/new_relic/agent/transaction.rb', line 556
def commit!(outermost_node_name)
generate_payload
assign_intrinsics
finalize_segments
@transaction_trace = transaction_sampler.on_finishing_transaction(self)
sql_sampler.on_finishing_transaction(state, @frozen_name)
record_summary_metrics(outermost_node_name)
record_total_time_metrics
record_apdex unless ignore_apdex?
record_queue_time
distributed_tracer.record_metrics
record_exceptions
record_transaction_event
record_log_events
merge_metrics
send_transaction_finished_event
end
|
#cpu_burn ⇒ Object
889
890
891
|
# File 'lib/new_relic/agent/transaction.rb', line 889
def cpu_burn
normal_cpu_burn || jruby_cpu_burn
end
|
#create_initial_segment(options = {}) ⇒ Object
446
447
448
449
|
# File 'lib/new_relic/agent/transaction.rb', line 446
def create_initial_segment(options = {})
segment = create_segment(@default_name, options)
segment.record_scoped_metric = false
end
|
#create_nested_segment(category, options) ⇒ Object
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
|
# File 'lib/new_relic/agent/transaction.rb', line 472
def create_nested_segment(category, options)
if options[:filtered_params] && !options[:filtered_params].empty?
@filtered_params = options[:filtered_params]
merge_request_parameters(options[:filtered_params])
end
@ignore_apdex = options[:ignore_apdex] if options.key?(:ignore_apdex)
@ignore_enduser = options[:ignore_enduser] if options.key?(:ignore_enduser)
nest_initial_segment if segments.length == 1
nested_name = self.class.nested_transaction_name(options[:transaction_name])
segment = create_segment(nested_name, options)
set_default_transaction_name(options[:transaction_name], category)
segment
end
|
#create_segment(name, options = {}) ⇒ Object
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
|
# File 'lib/new_relic/agent/transaction.rb', line 451
def create_segment(name, options = {})
summary_metrics = nil
if name.start_with?(MIDDLEWARE_PREFIX)
summary_metrics = MIDDLEWARE_SUMMARY_METRICS
end
@nesting_max_depth += 1
segment = Tracer.start_segment(
name: name,
unscoped_metrics: summary_metrics
)
segment.code_information = options
segment
end
|
#current_segment ⇒ Object
273
274
275
276
277
|
# File 'lib/new_relic/agent/transaction.rb', line 273
def current_segment
current_segment_by_thread[current_segment_key] ||
current_segment_by_thread[parent_segment_key] ||
current_segment_by_thread[@starting_segment_key]
end
|
#current_segment_key ⇒ Object
#default_name=(name) ⇒ Object
346
347
348
|
# File 'lib/new_relic/agent/transaction.rb', line 346
def default_name=(name)
@default_name = Helper.correctly_encoded(name)
end
|
#distributed_tracer ⇒ Object
287
288
289
|
# File 'lib/new_relic/agent/transaction.rb', line 287
def distributed_tracer
@distributed_tracer ||= DistributedTracer.new(self)
end
|
#finish ⇒ Object
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
|
# File 'lib/new_relic/agent/transaction.rb', line 521
def finish
return unless state.is_execution_traced?
@end_time = Process.clock_gettime(Process::CLOCK_REALTIME)
@duration = @end_time - @start_time
freeze_name_and_execute_if_not_ignored
if nesting_max_depth == 1
initial_segment.name = @frozen_name
end
initial_segment.transaction_name = @frozen_name
assign_segment_dt_attributes
assign_agent_attributes
initial_segment.finish
NewRelic::Agent::TransactionTimeAggregator.transaction_stop(@end_time, @starting_thread_id)
commit!(initial_segment.name) unless @ignore_this_transaction
rescue => e
NewRelic::Agent.logger.error('Exception during Transaction#finish', e)
nil
ensure
state.reset
end
|
#finished? ⇒ Boolean
442
443
444
|
# File 'lib/new_relic/agent/transaction.rb', line 442
def finished?
initial_segment&.finished?
end
|
#freeze_name_and_execute_if_not_ignored ⇒ Object
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
|
# File 'lib/new_relic/agent/transaction.rb', line 398
def freeze_name_and_execute_if_not_ignored
if !name_frozen?
name = promoted_transaction_name(best_name)
name = NewRelic::Agent.instance.transaction_rules.rename(name)
@name_frozen = true
if name.nil?
ignore!
@frozen_name = best_name
else
@frozen_name = name
end
end
if block_given? && !@ignore_this_transaction
yield
end
end
|
#generate_payload ⇒ Object
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
|
# File 'lib/new_relic/agent/transaction.rb', line 659
def generate_payload
@payload = {
:name => @frozen_name,
:bucket => recording_web_transaction? ? :request : :background,
:start_timestamp => start_time,
:duration => duration,
:metrics => @metrics,
:attributes => @attributes,
:error => false,
:priority => priority
}
distributed_tracer.append_payload(@payload)
append_apdex_perf_zone(@payload)
append_synthetics_to(@payload)
end
|
#had_error_affecting_apdex? ⇒ Boolean
794
795
796
797
798
799
800
801
802
|
# File 'lib/new_relic/agent/transaction.rb', line 794
def had_error_affecting_apdex?
@exceptions.each do |exception, options|
ignored = NewRelic::Agent.instance.error_collector.error_is_ignored?(exception)
expected = options[:expected]
return true unless ignored || expected
end
false
end
|
#ignore! ⇒ Object
905
906
907
|
# File 'lib/new_relic/agent/transaction.rb', line 905
def ignore!
@ignore_this_transaction = true
end
|
#ignore? ⇒ Boolean
909
910
911
|
# File 'lib/new_relic/agent/transaction.rb', line 909
def ignore?
@ignore_this_transaction
end
|
#ignore_apdex! ⇒ Object
913
914
915
|
# File 'lib/new_relic/agent/transaction.rb', line 913
def ignore_apdex!
@ignore_apdex = true
end
|
#ignore_apdex? ⇒ Boolean
917
918
919
|
# File 'lib/new_relic/agent/transaction.rb', line 917
def ignore_apdex?
@ignore_apdex
end
|
#ignore_enduser! ⇒ Object
921
922
923
|
# File 'lib/new_relic/agent/transaction.rb', line 921
def ignore_enduser!
@ignore_enduser = true
end
|
#ignore_enduser? ⇒ Boolean
925
926
927
|
# File 'lib/new_relic/agent/transaction.rb', line 925
def ignore_enduser?
@ignore_enduser
end
|
#ignore_trace? ⇒ Boolean
929
930
931
|
# File 'lib/new_relic/agent/transaction.rb', line 929
def ignore_trace?
@ignore_trace
end
|
#include_guid? ⇒ Boolean
676
677
678
|
# File 'lib/new_relic/agent/transaction.rb', line 676
def include_guid?
distributed_tracer.is_cross_app? || is_synthetics_request?
end
|
#influences_transaction_name?(category) ⇒ Boolean
375
376
377
|
# File 'lib/new_relic/agent/transaction.rb', line 375
def influences_transaction_name?(category)
!category || nesting_max_depth == 1 || similar_category?(category)
end
|
#initial_segment ⇒ Object
438
439
440
|
# File 'lib/new_relic/agent/transaction.rb', line 438
def initial_segment
segments.first
end
|
#instrumentation_state ⇒ Object
This transaction-local hash may be used as temporary storage by instrumentation that needs to pass data from one instrumentation point to another.
For example, if both A and B are instrumented, and A calls B but some piece of state needed by the instrumentation at B is only available at A, the instrumentation at A may write into the hash, call through, and then remove the key afterwards, allowing the instrumentation at B to read the value in between.
Keys should be symbols, and care should be taken to not generate key names dynamically, and to ensure that keys are removed upon return from the method that creates them.
338
339
340
|
# File 'lib/new_relic/agent/transaction.rb', line 338
def instrumentation_state
@instrumentation_state ||= {}
end
|
#is_synthetics_request? ⇒ Boolean
680
681
682
|
# File 'lib/new_relic/agent/transaction.rb', line 680
def is_synthetics_request?
!synthetics_payload.nil? && !.nil?
end
|
#jruby_cpu_burn ⇒ Object
899
900
901
902
903
|
# File 'lib/new_relic/agent/transaction.rb', line 899
def jruby_cpu_burn
return unless @jruby_cpu_start
jruby_cpu_time - @jruby_cpu_start
end
|
#log_frozen_name(name) ⇒ Object
370
371
372
373
|
# File 'lib/new_relic/agent/transaction.rb', line 370
def log_frozen_name(name)
NewRelic::Agent.logger.warn("Attempted to rename transaction to '#{name}' after transaction name was already frozen as '#{@frozen_name}'.")
nil
end
|
#merge_metrics ⇒ Object
734
735
736
|
# File 'lib/new_relic/agent/transaction.rb', line 734
def merge_metrics
NewRelic::Agent.instance.stats_engine.merge_transaction_metrics!(@metrics, best_name)
end
|
#merge_request_parameters(params) ⇒ Object
350
351
352
|
# File 'lib/new_relic/agent/transaction.rb', line 350
def merge_request_parameters(params)
merge_untrusted_agent_attributes(params, :'request.parameters', AttributeFilter::DST_NONE)
end
|
#merge_untrusted_agent_attributes(attributes, prefix, default_destinations) ⇒ Object
196
197
198
199
|
# File 'lib/new_relic/agent/transaction.rb', line 196
def merge_untrusted_agent_attributes(attributes, prefix, default_destinations)
@attributes.merge_untrusted_agent_attributes(attributes, prefix, default_destinations)
current_segment&.merge_untrusted_agent_attributes(attributes, prefix, default_destinations)
end
|
#name_frozen? ⇒ Boolean
417
418
419
|
# File 'lib/new_relic/agent/transaction.rb', line 417
def name_frozen?
@frozen_name ? true : false
end
|
#needs_middleware_summary_metrics?(name) ⇒ Boolean
517
518
519
|
# File 'lib/new_relic/agent/transaction.rb', line 517
def needs_middleware_summary_metrics?(name)
name.start_with?(MIDDLEWARE_PREFIX)
end
|
#nest_initial_segment ⇒ Object
489
490
491
492
|
# File 'lib/new_relic/agent/transaction.rb', line 489
def nest_initial_segment
self.initial_segment.name = self.class.nested_transaction_name(initial_segment.name)
initial_segment.record_scoped_metric = true
end
|
#normal_cpu_burn ⇒ Object
893
894
895
896
897
|
# File 'lib/new_relic/agent/transaction.rb', line 893
def normal_cpu_burn
return unless @process_cpu_start
process_cpu - @process_cpu_start
end
|
#notice_error(error, options = {}) ⇒ Object
Do not call this. Invoke the class method instead.
757
758
759
760
761
762
763
764
765
766
767
768
769
|
# File 'lib/new_relic/agent/transaction.rb', line 757
def notice_error(error, options = {}) if current_segment
current_segment.notice_error(error, expected: options[:expected])
options[:span_id] = current_segment.guid
end
if @exceptions[error]
@exceptions[error].merge!(options)
else
@exceptions[error] = options
end
end
|
#overridden_name=(name) ⇒ Object
342
343
344
|
# File 'lib/new_relic/agent/transaction.rb', line 342
def overridden_name=(name)
@overridden_name = Helper.correctly_encoded(name)
end
|
#parent_segment_key ⇒ Object
269
270
271
|
# File 'lib/new_relic/agent/transaction.rb', line 269
def parent_segment_key
(::Fiber.current.nr_parent_key if ::Fiber.current.respond_to?(:nr_parent_key)) || (::Thread.current.nr_parent_key if ::Thread.current.respond_to?(:nr_parent_key))
end
|
390
391
392
393
394
395
396
|
# File 'lib/new_relic/agent/transaction.rb', line 390
def promoted_transaction_name(name)
if name.start_with?(MIDDLEWARE_PREFIX)
"#{CONTROLLER_PREFIX}#{name}"
else
name
end
end
|
#queue_time ⇒ Object
779
780
781
|
# File 'lib/new_relic/agent/transaction.rb', line 779
def queue_time
@apdex_start ? @start_time - @apdex_start : 0
end
|
#record_apdex ⇒ Object
808
809
810
811
812
813
814
815
816
817
818
819
820
|
# File 'lib/new_relic/agent/transaction.rb', line 808
def record_apdex
return unless state.is_execution_traced?
freeze_name_and_execute_if_not_ignored do
if recording_web_transaction?
record_apdex_metrics(APDEX_METRIC, APDEX_TXN_METRIC_PREFIX, apdex_t)
else
record_apdex_metrics(APDEX_OTHER_METRIC,
APDEX_OTHER_TXN_METRIC_PREFIX,
transaction_specific_apdex_t)
end
end
end
|
#record_apdex_metrics(rollup_metric, transaction_prefix, current_apdex_t) ⇒ Object
822
823
824
825
826
827
828
829
830
831
832
833
|
# File 'lib/new_relic/agent/transaction.rb', line 822
def record_apdex_metrics(rollup_metric, transaction_prefix, current_apdex_t)
return unless current_apdex_t
total_duration = end_time - apdex_start
apdex_bucket_global = apdex_bucket(total_duration, current_apdex_t)
apdex_bucket_txn = apdex_bucket(duration, current_apdex_t)
@metrics.record_unscoped(rollup_metric, apdex_bucket_global, current_apdex_t)
@metrics.record_unscoped(APDEX_ALL_METRIC, apdex_bucket_global, current_apdex_t)
txn_apdex_metric = @frozen_name.sub(/^[^\/]+\//, transaction_prefix)
@metrics.record_unscoped(txn_apdex_metric, apdex_bucket_txn, current_apdex_t)
end
|
#record_exception(exception, options, error_recorded) ⇒ Object
746
747
748
749
750
751
752
753
754
|
# File 'lib/new_relic/agent/transaction.rb', line 746
def record_exception(exception, options, error_recorded)
options[:uri] ||= request_path if request_path
options[:port] = request_port if request_port
options[:metric] = best_name
options[:attributes] = @attributes
span_id = options.delete(:span_id)
!!agent.error_collector.notice_error(exception, options, span_id) || error_recorded
end
|
#record_exceptions ⇒ Object
738
739
740
741
742
743
744
|
# File 'lib/new_relic/agent/transaction.rb', line 738
def record_exceptions
error_recorded = false
@exceptions.each do |exception, options|
error_recorded = record_exception(exception, options, error_recorded)
end
payload&.[]=(:error, error_recorded)
end
|
#record_log_events ⇒ Object
775
776
777
|
# File 'lib/new_relic/agent/transaction.rb', line 775
def record_log_events
agent.log_event_aggregator.record_batch(self, @logs.to_a)
end
|
#record_queue_time ⇒ Object
#record_summary_metrics(outermost_node_name) ⇒ Object
The summary metrics recorded by this method all end up with a duration equal to the transaction itself, and an exclusive time of zero.
647
648
649
650
651
|
# File 'lib/new_relic/agent/transaction.rb', line 647
def record_summary_metrics(outermost_node_name)
metrics = summary_metrics
metrics << @frozen_name unless @frozen_name == outermost_node_name
@metrics.record_unscoped(metrics, duration, 0)
end
|
#record_transaction_event ⇒ Object
771
772
773
|
# File 'lib/new_relic/agent/transaction.rb', line 771
def record_transaction_event
agent.transaction_event_recorder.record(payload)
end
|
#recording_web_transaction? ⇒ Boolean
877
878
879
|
# File 'lib/new_relic/agent/transaction.rb', line 877
def recording_web_transaction?
web_category?(@category)
end
|
#referer ⇒ Object
312
313
314
|
# File 'lib/new_relic/agent/transaction.rb', line 312
def referer
@request_attributes&.referer
end
|
#remove_current_segment_by_thread_id(id) ⇒ Object
283
284
285
|
# File 'lib/new_relic/agent/transaction.rb', line 283
def remove_current_segment_by_thread_id(id)
@current_segment_lock.synchronize { current_segment_by_thread.delete(id) }
end
|
#request_path ⇒ Object
316
317
318
|
# File 'lib/new_relic/agent/transaction.rb', line 316
def request_path
@request_attributes&.request_path
end
|
#request_port ⇒ Object
320
321
322
|
# File 'lib/new_relic/agent/transaction.rb', line 320
def request_port
@request_attributes&.port
end
|
#sampled? ⇒ Boolean
291
292
293
294
295
296
297
298
|
# File 'lib/new_relic/agent/transaction.rb', line 291
def sampled?
return unless Agent.config[:'distributed_tracing.enabled']
if @sampled.nil?
@sampled = NewRelic::Agent.instance.adaptive_sampler.sampled?
end
@sampled
end
|
#send_transaction_finished_event ⇒ Object
This event is fired when the transaction is fully completed. The metric values and sampler can’t be successfully modified from this event.
655
656
657
|
# File 'lib/new_relic/agent/transaction.rb', line 655
def send_transaction_finished_event
agent.events.notify(:transaction_finished, payload)
end
|
#set_current_segment(new_segment) ⇒ Object
279
280
281
|
# File 'lib/new_relic/agent/transaction.rb', line 279
def set_current_segment(new_segment)
@current_segment_lock.synchronize { current_segment_by_thread[current_segment_key] = new_segment }
end
|
#set_default_transaction_name(name, category) ⇒ Object
354
355
356
357
358
359
360
361
|
# File 'lib/new_relic/agent/transaction.rb', line 354
def set_default_transaction_name(name, category)
return log_frozen_name(name) if name_frozen?
if influences_transaction_name?(category)
self.default_name = name
@category = category if category
end
end
|
#set_overriding_transaction_name(name, category) ⇒ Object
363
364
365
366
367
368
|
# File 'lib/new_relic/agent/transaction.rb', line 363
def set_overriding_transaction_name(name, category)
return log_frozen_name(name) if name_frozen?
self.overridden_name = name
@category = category if category
end
|
#similar_category?(category) ⇒ Boolean
885
886
887
|
# File 'lib/new_relic/agent/transaction.rb', line 885
def similar_category?(category)
web_category?(@category) == web_category?(category)
end
|
#start(options = {}) ⇒ Object
#summary_metrics ⇒ Object
500
501
502
503
504
505
506
|
# File 'lib/new_relic/agent/transaction.rb', line 500
def summary_metrics
if @frozen_name.start_with?(CONTROLLER_PREFIX)
[WEB_SUMMARY_METRIC]
else
background_summary_metrics
end
end
|
#synthetics_account_id ⇒ Object
689
690
691
692
|
# File 'lib/new_relic/agent/transaction.rb', line 689
def synthetics_account_id
info = synthetics_payload or return nil
info[1]
end
|
#synthetics_job_id ⇒ Object
699
700
701
702
|
# File 'lib/new_relic/agent/transaction.rb', line 699
def synthetics_job_id
info = synthetics_payload or return nil
info[3]
end
|
#synthetics_monitor_id ⇒ Object
704
705
706
707
|
# File 'lib/new_relic/agent/transaction.rb', line 704
def synthetics_monitor_id
info = synthetics_payload or return nil
info[4]
end
|
#synthetics_resource_id ⇒ Object
694
695
696
697
|
# File 'lib/new_relic/agent/transaction.rb', line 694
def synthetics_resource_id
info = synthetics_payload or return nil
info[2]
end
|
#synthetics_version ⇒ Object
684
685
686
687
|
# File 'lib/new_relic/agent/transaction.rb', line 684
def synthetics_version
info = synthetics_payload or return nil
info[0]
end
|
#threshold ⇒ Object
844
845
846
847
848
849
850
851
|
# File 'lib/new_relic/agent/transaction.rb', line 844
def threshold
source_class = Agent.config.source(:'transaction_tracer.transaction_threshold').class
if source_class == Configuration::DefaultSource
apdex_t * 4
else
Agent.config[:'transaction_tracer.transaction_threshold']
end
end
|
#trace_id=(value) ⇒ Object
304
305
306
|
# File 'lib/new_relic/agent/transaction.rb', line 304
def trace_id=(value)
@trace_id = value
end
|
#transaction_specific_apdex_t ⇒ Object
839
840
841
842
|
# File 'lib/new_relic/agent/transaction.rb', line 839
def transaction_specific_apdex_t
key = :web_transactions_apdex
Agent.config[key] && Agent.config[key][best_name]
end
|
#user_defined_rules_ignore? ⇒ Boolean
547
548
549
550
551
552
553
554
|
# File 'lib/new_relic/agent/transaction.rb', line 547
def user_defined_rules_ignore?
return unless request_path
return if (rules = NewRelic::Agent.config[:"rules.ignore_url_regexes"]).empty?
rules.any? do |rule|
request_path.match(rule)
end
end
|
#web_category?(category) ⇒ Boolean
881
882
883
|
# File 'lib/new_relic/agent/transaction.rb', line 881
def web_category?(category)
WEB_TRANSACTION_CATEGORIES.include?(category)
end
|
#with_database_metric_name(model, method, product = nil) ⇒ Object
853
854
855
856
857
858
859
860
861
862
863
864
865
866
867
|
# File 'lib/new_relic/agent/transaction.rb', line 853
def with_database_metric_name(model, method, product = nil)
previous = self.instrumentation_state[:datastore_override]
model_name = case model
when Class
model.name
when String
model
else
model.to_s
end
self.instrumentation_state[:datastore_override] = [method, model_name, product]
yield
ensure
self.instrumentation_state[:datastore_override] = previous
end
|